#include "stdafx.h"


#include "function.h"


#include "function.h"

void rG(gsl_rng *r, double *mu, double *v, double m, double s, double S, double tau, double y){
	double Si = S + pow(y - m, 2.0) / (1.0 + tau);
	double X = tau / (1.0 + tau);
	double xi = (m + tau*y) / (1.0 + tau);

	*v = 1.0 / (gsl_ran_gamma(r, (1.0 + s) / 2.0, 2.0 / Si));
	*mu = xi + gsl_ran_gaussian(r, sqrt(X*(*v)));
	return;
}

// not normalized q
void compute_q(double *q, int *n_j, double alpha, double m, double s, double S, double tau, double y, int K, double *mu, double *V, int *rindex){
	double M = (1.0 + tau)*S / s;
	double c = gsl_sf_gamma((1.0 + s) / 2.0) / gsl_sf_gamma(s / 2.0) / sqrt(s);
	int ind;
	q[K] = alpha * c * pow(1.0 + pow(y - m, 2.0) / (s*M), -(1.0 + s) / 2.0) / sqrt(M);
	for (int j = 0; j<K; j++){
		ind = rindex[j];
		q[j] = (double)n_j[ind] * exp(-pow(y - mu[ind], 2.0) / (2.0*V[ind])) / sqrt(2.0*V[ind]);
	}
	return;
}

int rbernoulli(gsl_rng *r, double prob)	{
	if (gsl_ran_flat(r, 0.0, 1.0) < prob)
		return(1);
	else
		return(0);
}

int rmultinom(gsl_rng *r, double *prob, int length, bool normalized){

	double p = gsl_ran_flat(r, 0.0, 1.0), cum = 0.0, p_sum = 0.0;

	if (normalized == false)	{
		for (int k = 0; k<length; k++)
			p_sum += prob[k];
		for (int k = 0; k<length; k++)
			prob[k] = prob[k] / p_sum;
	}

	for (int k = 0; k<length - 1; k++)	{
		cum += prob[k];
		if (p < cum)
			return(k);
	}
	return(length - 1);
}

void swap(int *index, int *rindex, int i, int j){
	int tmp;
	tmp = index[i];
	index[i] = index[j];
	index[j] = tmp;

	tmp = rindex[index[i]];
	rindex[index[i]] = rindex[index[j]];
	rindex[index[j]] = tmp;
	return;
}

double rm(gsl_rng *r, double tau, double u, double U, int K, double *mu, double *V, int *rindex){
	double sum1 = 0.0, sum2 = 0.0;
	int i;
	for (i = 0; i<K; i++){
		sum1 += 1.0 / V[rindex[i]];
		sum2 += mu[rindex[i]] / V[rindex[i]];
	}
	double x = U / (U + tau / sum1);
	return((1 - x) * u + x * sum2 / sum1 + gsl_ran_gaussian(r, sqrt(x * tau / sum1)));
}

double rtau(gsl_rng *r, double m, double w, double W, int K, double *mu, double *V, int *rindex){
	double sum = 0.0;
	int i;
	for (i = 0; i<K; i++)	sum += pow(mu[rindex[i]] - m, 2.0) / V[rindex[i]];
	return(1.0 / gsl_ran_gamma(r, (w + (double)K) / 2.0, 2.0 / (W + sum)));
}

double ralpha(gsl_rng *r, double eta, int n, int K, double a, double b){
	if (rbernoulli(r, 1.0 / (1.0 + (double)n*(b - eta) / (a + (double)K - 1.0))) == 1) {
		return(gsl_ran_gamma(r, a + (double)K, 1.0 / (b - eta)));
	}
	else {
		return(gsl_ran_gamma(r, a + (double)K - 1.0, 1.0 / (b - eta)));
	}
}

int compare(const void* pa, const void* pb){
	if ((*(double*)pa) - (*(double*)pb) > 0.0)
		return(1);
	else
		return(-1);
}

void acf(double *acf_a, double *a, int N){
	double sum1 = 0.0;
	double sum2 = 0.0;
	double m = 0.0;
	for (int i = 0; i<N; i++){
		m += a[i];
	}
	m /= (double)N;

	for (int i = 0; i<N; i++){
		sum1 += pow(a[i] - m, 2.0);
	}

	acf_a[0] = 1.0;
	for (int k = 1; k<30; k++){
		sum2 = 0.0;
		for (int i = 0; i<N - k; i++){
			sum2 += (a[i] - m) * (a[i + k] - m);
		}
		acf_a[k] = sum2 / sum1;
	}
	return;
}

double rand_range(double fr, double to)
{
	double ret = (double)rand() / (RAND_MAX + 1) * (to - fr) + fr;
	return ret;
}

void linespace(double* arr, double fr, double to, int nr_data)
{
	for (int i = 0; i < nr_data; i++)
	{
		double val;
		val = fr + (to - fr)*i / nr_data;
		arr[i] = val;
	}
}